//////////////////////////////////////////
// BCC2ECCProc.h
// Copyright (c) 09/06/2022 by ECCSkelton
//////////////////////////////////////////

/////////////////////////////////
//主ウィンドウCMyWndの関数の定義
//ウィンドウメッセージ関数
/////////////////////////////////

bool CMyWnd::OnClose(WPARAM wParam, LPARAM lParam) {

	if(MessageBoxW(m_hWnd, L"終了しますか", L"終了確認",
					MB_YESNO | MB_ICONQUESTION) == IDYES)
		//処理をするとDestroyWindow、PostQuitMessageが呼ばれる
		return TRUE;
	else
		//そうでなければウィンドウではDefWindowProc関数をreturn、ダイアログではreturn FALSEとなる。
		return FALSE;
}

//ダイアログベースの場合はこれが必要
bool CMyWnd::OnDestroy(WPARAM wPram, LPARAM lParam) {

	PostQuitMessage(0);
	return TRUE;
}

/////////////////////////////////
//主ウィンドウCMyWndの関数の定義
//メニュー項目、コントロール関数
/////////////////////////////////
bool CMyWnd::OnSel() {

	WCHAR* fn = cmndlg.GetFileName(m_hWnd, L"BDPファイル(*.bdp)\0*.bdp\0\0",
						TRUE, L"bdp", L".", L"プロジェクトファイルを開く");
	if(!fn) {
		MessageBoxW(m_hWnd, L"ファイルが選択されませんでした。", L"エラー",
					MB_OK | MB_ICONERROR);
		return FALSE;
	}
	else {
		//フルパスプロジェクトファイル名を記録
		lstrcpy(m_PathName, fn);
		m_Ext = StrStrW(m_PathName, L".bdp");	//拡張子の位置を記録
		//フルパスプロジェクトファイル名
		WCHAR* cp = m_PathName;
		while(cp < m_Ext) {
			if(*cp == L'\\')
				fn = cp;					//（最後の）'\\'の位置を記録
			cp++;
		}
		++fn;								//'\\'の次に進める
		for(cp = fn; cp < m_Ext; cp++)
			m_PrjtName[cp - fn] = *cp;		//プロジェクト名の保存
		SendItemMsg(IDC_EDIT, WM_SETTEXT, 0, (LPARAM)m_PrjtName);
		MessageBoxW(m_hWnd, L"選択されたBCCSkeltonプロジェクトのrcファイル、"\
					"cppフィル、hファイルおよびProc.hファイルを順に"\
					"ECCSkelton用に変換します。\r\n但し、メインウィンドウで"\
					"ないダイアログがあれば、コマンドテーブル（CMDTABLE）の"\
					"クラス名とテーブルをマニュアルで修正する必要があります。",
					L"説明", MB_OK | MB_ICONINFORMATION);
		return TRUE;
	}
}

bool CMyWnd::OnIdok() {

	if(!*m_PathName) {
		MessageBoxW(m_hWnd, L"未だファイルが選択されていません。", L"エラー"	,
					MB_OK | MB_ICONERROR);
		return FALSE;
	}
	//チェックボックスをチェックする
	if(SendItemMsg(IDC_OPTION, BM_GETCHECK, 0, (LPARAM)0) == BST_CHECKED)
		m_Option = TRUE;
	//RCファイルの変更
	lstrcpyW(StrStrW(m_PathName, L".bdp"), L".rc");
	ModifyRC();
	//CPPファイルの変更
	lstrcpyW(StrStrW(m_PathName, L".rc"), L".cpp");
	ModifyCPP();
	//CPPファイルの変更
	lstrcpyW(StrStrW(m_PathName, L".cpp"), L".h");
	ModifyH();
	//CPPファイルの変更
	lstrcpyW(StrStrW(m_PathName, L".h"), L"Proc.h");
	ModifyProcH();
	//再変換、再初期化、または終了する
	int res = MessageBoxW(m_hWnd, L"変換処理されたECCSkelton用ファイルは"\
						"\"_ECC\"が付いたファイルになっています。\r\n内容を"\
						"確認して再度変換する場合は「はい」を、再初期化する"\
						"場合は「いいえ」を、終了する場合は「キャンセル」を"\
						"押してください。",
						L"続行確認", MB_YESNOCANCEL | MB_ICONQUESTION);
	if(res == IDNO)
		Init();
	else if(res == IDCANCEL)
		OnIdno();
	else
		return TRUE;
}

bool CMyWnd::OnIdno() {

	SendMsg(WM_CLOSE, 0, 0);
	return TRUE;
}

////////////////////
//ユーザー関数定義
///////////////////
void CMyWnd::Init() {

	*m_PathName = L'\0';
	m_Ext = L'\0';
	*m_PrjtName = L'\0';
	m_Option = FALSE;
	//エディットボックスを空にする
	SendItemMsg(IDC_EDIT, WM_SETTEXT, 0, (LPARAM)L"");
	//リストボックスを空にする
	while(SendItemMsg(IDC_LISTBOX, LB_DELETESTRING, 0, 0));
	//チェックボックスを未チェックにする
	SendItemMsg(IDC_OPTION, BM_SETCHECK, BST_UNCHECKED, (LPARAM)0);
}

bool CMyWnd::ModifyRC() {

	CSTR File, fn(m_PrjtName);
	File.FromFile(m_PathName); 
	fn = fn + L".rc を処理します。";
	SendItemMsg(IDC_LISTBOX, LB_ADDSTRING, 0, (LPARAM)fn.ToChar());
	//共通
	File.ReplaceAll(L", \"", L", L\"");	//[, "]→[, L"]
	//ダイアログ
	File.ReplaceAll(L"CAPTION \"", L"CAPTION L\"");	//[CAPTION "]→[CAPTION L"]
	File.ReplaceAll(L"CONTROL \"", L"CONTROL L\"");	//[CONTROL "]→[CONTROL L"]
	//メニュー
	File.ReplaceAll(L"POPUP \"", L"POPUP L\"");		//[POPUP "]→[POPUP L"]
	File.ReplaceAll(L"MENUITEM \"", L"MENUITEM L\"");	//[MENUITEM "]→[POPUP L"]
	*m_Ext = L'\0';
	fn = m_PathName;
	*m_Ext = L'.';
	fn = fn + L"_ECC.rc";
	File.ToSJISFile(fn.ToChar());
	return TRUE;
}

bool CMyWnd::ModifyCPP() {

	CSTR File, fn(m_PrjtName);
	File.FromFile(m_PathName); 
	fn = fn + L".cpp を処理します。";
	SendItemMsg(IDC_LISTBOX, LB_ADDSTRING, 0, (LPARAM)fn.ToChar());
	File.ReplaceAll(L"BCCSkelton", L"ECCSkelton");
	File.ReplaceAll(L"WinMain(", L"wWinMain(");	//[WinMain(]→[wWinMain(]
	File.ReplaceAll(L"LPSTR", L"LPWSTR");			//[LPSTR]→[LPWSTR]
	File.ReplaceAll(L", SDIPROC", L"");			//[, SDIPROC]→[]
	File.ReplaceAll(L", ModelessProc", L"");		//[, ModelessProc]→[]
	File.ReplaceAll(L"(\"", L"(L\"");				//[("]→[(L"]
	File.ReplaceAll(L" \"", L" L\"");				//[ "]→[ L"]
	*m_Ext = L'\0';
	fn = m_PathName;
	*m_Ext = L'.';
	fn = fn + L"_ECC.cpp";
	File.ToSJISFile(fn.ToChar());
	return TRUE;
}

bool CMyWnd::ModifyH() {

	CSTR File, fn(m_PrjtName), work(L"ON_COMMAND(");
	File.FromFile(m_PathName); 
	fn = fn + L".h を処理します。";
	SendItemMsg(IDC_LISTBOX, LB_ADDSTRING, 0, (LPARAM)fn.ToChar());
	//共通
	File.ReplaceAll(L"BCCSkelton", L"ECCSkelton");
	//メッセージ、コマンドテーブル共通
	work = work + m_PrjtName;
	work = work + L", ";
	File.ReplaceAll(work.ToChar(), L"ON(");			//[ON_COMMAND(]→[ON(]
	File.ReplaceAll(L"ON_(", L"//ON_(");			//[ON_(Win message)]→[//ON_(Win message)]
	File.ReplaceAll(L"ON_", L"//ON_");				//[ON_]→[//ON_]
	File.ReplaceAll(L"(\"", L"(L\"");				//[("]→[(L"]
	File.ReplaceAll(L" \"", L" L\"");				//[ "]→[ L"]
	File.ReplaceAll(L"char", L"WCHAR");				//[char]→[WCHAR]
	//メインウィンドウのメンバー関数にCMDTABLEを追加
	File.ReplaceAll(L"//ウィンドウメッセージ関連\r\n\tbool",
					L"//ウィンドウメッセージ関連\r\n\tCMDTABLE\t\t//OnCommand()関数宣言\r\n\tbool");
	//CSDIウィンドウの場合
	File.ReplaceAll(L"BEGIN_SDIMSG(",
					L"BEGIN_CMDTABLE(CMyWnd)\t//クラス名がCMyWndではない場合、クラス名、テーブルを適宜マニュアルで修正してください\r\n//BEGIN_SDIMSG(");
	File.ReplaceAll(L"END_WNDMSG",
					L"//END_WNDMSG\r\nEND_CMDTABLE");
	//CDLGベースのモードレスダイアログの場合
	File.ReplaceAll(L"BEGIN_MODELESSDLGMSG(",
					L"BEGIN_CMDTABLE(CMyWnd)\t//クラス名がCMyWndではない場合、クラス名、テーブルを適宜マニュアルで修正してください\r\n//BEGIN_MODELESSDLGMSG(");
	//CDLGベースのモーダルダイアログの場合
	File.ReplaceAll(L"BEGIN_MODALDLGMSG(",
					L"BEGIN_CMDTABLE(CMyWnd)\t//クラス名がCMyWndではない場合、クラス名、テーブルを適宜マニュアルで修正してください\r\n//BEGIN_MODALDLGMSG(");
	File.ReplaceAll(L" : public CDLG {\r\npublic:",
					L" : public CDLG {\r\npublic:\r\n\tCMDTABLE\t\t//OnCommand()関数宣言");
	//ダイアログ共通
	File.ReplaceAll(L"END_DLGMSG", L"//END_DLGMSG\r\nEND_CMDTABLE");
	*m_Ext = L'\0';
	fn = m_PathName;
	*m_Ext = L'.';
	fn = fn + L"_ECC.h";
	File.ToSJISFile(fn.ToChar());
	return TRUE;
}

bool CMyWnd::ModifyProcH() {

	CSTR File, fn(m_PrjtName);
	File.FromFile(m_PathName); 
	fn = fn + L"Proc.h を処理します。";
	SendItemMsg(IDC_LISTBOX, LB_ADDSTRING, 0, (LPARAM)fn.ToChar());
	File.ReplaceAll(L"BCCSkelton", L"ECCSkelton");
	File.ReplaceAll(L" \"", L" L\"");			//[ "]→[ L"]
	File.ReplaceAll(L" \'", L" L\'");			//[ ']→[ L']
	File.ReplaceAll(L"char", L"WCHAR");			//[char]→[WCHAR]
	File.ReplaceAll(L"LPSTR", L"LPWSTR");		//[LPSTR]→[LPWSTR]
	File.ReplaceAll(L"LPCSTR", L"LPCWSTR");		//[LPCSTR]→[LPCWSTR]
	///////////////////////////////////////////////////////////////////
	//既にプログラムを書き始めている場合の為に以下オプションを用意した。
	//多くのWin32API関数名はUNICODEが定義されていれば、ワイド文字にWを
	//付けてくれる為、特に変換しなくてもよい。
	//なお、自分用にもっと増やしたい場合は↓のリストに追加すればよい。
	///////////////////////////////////////////////////////////////////
	if(m_Option) {
		//Win32 API関数
		File.ReplaceAll(L"MessageBox", L"MessageBoxW");		//[MessageBox]→[MessageBoxW]
		File.ReplaceAll(L"SendMessage", L"SendMessageW");	//[SendMessage]→[SendMessageW]
		File.ReplaceAll(L"SendDlgItemMessage", L"SendDlgItemMessageW");	//[SendDlgItemMessage]→[SendDlgItemMessageW]
		File.ReplaceAll(L"lstrcmp", L"lstrcmpW");			//[lstrcmp]→[lstrcmpW]
		File.ReplaceAll(L"lstrcpy", L"lstrcpyW");			//[lstrcpy]→[lstrcpyW]
		File.ReplaceAll(L"lstrcat", L"lstrcatW");			//[lstrcat]→[lstrcatW]
		File.ReplaceAll(L"lstrlen", L"lstrlenW");			//[lstrlen]→[lstrlenW]
		File.ReplaceAll(L"wsprintf", L"wsprintfW");			//[wsprintf]→[wsprintfW]
		//C標準ライブラリ―
		File.ReplaceAll(L"strstr", L"StrStrW");				//[strstr]→[StrStrW] Shellの関数を優先した
		//File.ReplaceAll(L"strstr", L"wcsstr");			//[strstr]→[wcsstr]
		File.ReplaceAll(L"strcmp", L"wcscmp");				//[strcmp]→[wcscmp]
		File.ReplaceAll(L"strcpy", L"wcscpy");				//[strcpy]→[wcscpy]
		File.ReplaceAll(L"strcat", L"wcscat");				//[strcat]→[wcscat]
		File.ReplaceAll(L"strlen", L"wcslen");				//[strlen]→[wcslen]
	}
	*m_Ext = L'\0';
	fn = m_PathName;
	*m_Ext = L'.';
	fn = fn + L"Proc_ECC.h";
	File.ToSJISFile(fn.ToChar());
	return TRUE;
}
